from corsheaders.signals import check_request_enabled

from django_jwt.server.models import WebPage


def cors_allow_webpages(sender, request, **kwargs):
    origin = request.META.get('HTTP_ORIGIN', None)
    if origin is not None:
        return WebPage.objects.filter(host=origin).exists()


check_request_enabled.connect(cors_allow_webpages)



from django.contrib.auth.models import AnonymousUser
from django.utils.deprecation import MiddlewareMixin

from django_jwt.auth import JWTAuthentication
from django_jwt.settings_utils import get_setting


class JWTAuthenticationMiddleware(MiddlewareMixin):
    def process_request(self, request):
        key = request.COOKIES.get(get_setting('JWT_CLIENT.COOKIE_NAME'), None)
        user = None
        if key is not None:
            try:
                user = JWTAuthentication.authenticate_credentials(key)
            except JWTAuthentication.JWTException:
                pass
        return user or AnonymousUser()



# Generated by Django 4.0 on 2022-02-22 11:05

import datetime
from django.db import migrations, models
import django.db.models.deletion
import django_jwt.server.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sessions', '0001_initial'),
        ('user', '0003_alter_person_email'),
    ]

    operations = [
        migrations.CreateModel(
            name='Key',
            fields=[
                ('kid', models.CharField(default=uuid.uuid4, editable=False, max_length=40, primary_key=True, serialize=False)),
                ('private_key', models.BinaryField(default=django_jwt.server.models.generate_key)),
                ('date', models.DateTimeField(default=datetime.datetime.now, editable=False)),
            ],
        ),
        migrations.CreateModel(
            name='WebPage',
            fields=[
                ('id', models.CharField(default=uuid.uuid4, editable=False, max_length=40, primary_key=True, serialize=False, verbose_name='Client id')),
                ('host', models.CharField(max_length=200, unique=True)),
                ('needs_confirmation', models.BooleanField(default=True)),
            ],
        ),
        migrations.CreateModel(
            name='UserWebPagePermission',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='user.user')),
                ('web', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='server.webpage')),
            ],
        ),
        migrations.CreateModel(
            name='UserExternalSession',
            fields=[
                ('id', models.CharField(default=uuid.uuid4, editable=False, max_length=40, primary_key=True, serialize=False)),
                ('extra_id', models.CharField(default=uuid.uuid4, max_length=40, unique=True)),
                ('date', models.DateTimeField(auto_now_add=True)),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sessions.session')),
                ('web', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='server.webpage')),
            ],
        ),
        migrations.CreateModel(
            name='AttributeWebPage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('attribute', models.CharField(max_length=100)),
                ('value', models.CharField(max_length=100)),
                ('restrict', models.BooleanField(default=False)),
                ('web', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='server.webpage')),
            ],
        ),
    ]



from django.urls import path

from django_jwt import views
from django_jwt.settings_utils import get_setting

urlpatterns = []

if get_setting('JWT_CLIENT.OPENID2_URL') == 'fake':
    urlpatterns.extend([
        path('jwks', views.jwks, name="fake_jwks"),
        path('fake-login', views.fake_login, name="fake_login"),
        path('.well-known/openid-configuration', views.fake_config, name='fake_config'),
        path('fake-userinfo', views.fake_userinfo, name='fake_userinfo'),
    ])



from django.apps import AppConfig

from django_jwt.settings_utils import get_setting


class DjangoJwtServerConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_jwt.server'

    def ready(self):
        apps = get_setting('INSTALLED_APPS')
        if 'corsheaders' in apps:
            from django_jwt.server import signals
            signals
        else:
            raise Exception('corsheaders is required in order to protect your app from CORS')



from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import PermissionDenied, SuspiciousOperation
from django.http import JsonResponse
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.http import urlencode
from django.views import View
from django.views.generic import TemplateView

from django_jwt.server.models import WebPage, UserWebPagePermission, UserExternalSession, Key
from django_jwt.server.views_utils import create_jwt_code, get_value


class OpenIdConfiguration(View):
    def get(self, request):
        configuration = {
            'issuer': request.build_absolute_uri('/'),
            'authorization_endpoint': request.build_absolute_uri(reverse('authorization_endpoint')),
            # 'userinfo_endpoint': request.build_absolute_uri(reverse('userinfo_endpoint')),
            'jwks_uri': request.build_absolute_uri(reverse('jwks_uri')),
            # 'end_session_endpoint': request.build_absolute_uri(reverse('end_session_endpoint')),
            'response_types_supported': ['id_token', 'token'],
            'subject_types_supported': ['public'],
            'id_token_signing_alg_values_supported': ['RS256'],
            'claims_supported': ['sub', 'iss', 'aud', 'exp', 'iat', 'jti', 'scope', 'azp'],
            'require_request_uri_registration': True,
        }
        return JsonResponse(configuration)


class AuthorizationView(LoginRequiredMixin, TemplateView):
    template_name = 'oidc_confirmation.html'

    def get_login_url(self):
        connection = self.request.GET.get('connection', '')
        return super().get_login_url() + (('-' + connection) if connection else '')

    def dispatch(self, request, *args, **kwargs):
        try:
            kwargs['web'] = WebPage.objects.prefetch_related('attributewebpage_set')\
                .get(id=self.request.GET.get('client_id'))
        except WebPage.DoesNotExist:
            raise PermissionDenied()
        redirect_url = self.request.GET.get('redirect_uri', None)
        if redirect_url is None or not redirect_url.startswith(kwargs['web'].host + '/'):
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        web = kwargs.get('web')
        self.check_user_permissions(web)
        if web.needs_confirmation:
            try:
                UserWebPagePermission.objects.get(web=web, user=request.user)
            except UserWebPagePermission.DoesNotExist:
                return render(request, self.template_name, context={'web': web})
        return self.success_response(web)

    def success_response(self, web):
        external_session = UserExternalSession.objects\
            .get_or_create(web=web, session_id=self.request.session.session_key)[0]
        tokens = self.request.GET.get('response_type', None)
        if tokens is None:
            raise SuspiciousOperation("Invalid request; see documentation for correct paramaters")
        tokens = tokens.split(' ')
        response_parameters = {'state': self.request.GET.get('state')}
        if 'token' in tokens:
            response_parameters['access_token'] = create_jwt_code(request=self.request, token='access_token',
                                                                  session=external_session)
        if 'id_token' in tokens:
            response_parameters['id_token'] = create_jwt_code(
                request=self.request, token='id_token', session=external_session,
                access_token=response_parameters.get('access_token', None))
        return redirect(self.request.GET.get('redirect_uri') + '#' + urlencode(response_parameters))

    def post(self, request, *args, **kwargs):
        web = kwargs.get('web')
        if request.POST.get('confirmation', 'false') == 'true':
            UserWebPagePermission(web=web, user=request.user).save()
            return redirect(request.get_full_path())
        return render(request, self.template_name, context={'web': web, 'denied': True})

    def check_user_permissions(self, web):
        for restrict in web.attributewebpage_set.filter(restrict=True):
            if str(get_value(self.request.user, restrict.attribute.split('.'))) == restrict.value:
                raise PermissionDenied('You don\'t have permission to log in into this web page.')


class JWKsView(View):
    def get(self, request):
        return JsonResponse({'keys': Key.get_jwk_set()})



import json

import urllib3
from django.urls import reverse
from jwcrypto.jwk import JWKSet

from django_jwt.exceptions import JWTClientException
from django_jwt.settings_utils import get_setting


# singleton class that saves all information from the OpenId server
class OpenId2Info:
    def __init__(self):
        url = get_setting('JWT_CLIENT.OPENID2_URL')
        if url == 'local':
            apps = get_setting('INSTALLED_APPS')
            if 'django_jwt.server' not in apps:
                raise JWTClientException('JWT_CLIENT.OPENID2_URL not set or django_jwt.server not installed')
            url = get_setting('DEFAULT_DOMAIN') + reverse('oidc_config')
        elif url == 'fake':
            url = get_setting('DEFAULT_DOMAIN') + reverse('fake_config')
        else:
            url += '/.well-known/openid-configuration'

        # Getting urls info from OpenId server
        http = urllib3.PoolManager()
        r = http.request('GET', url)
        if r.status != 200:
            raise JWTClientException('OpenID returned error code %s on openid-configuration: %s' % (r.status, url))
        data = json.loads(r.data.decode('UTF-8'))
        self.jwks_uri = data.get('jwks_uri', None)
        if self.jwks_uri is None:
            raise JWTClientException('jkws_uri not found in OpenId openid-configuration url')
        self.fetch_jwks()

    # Updates all the actual JWKS from the OPenId server
    def fetch_jwks(self):
        http = urllib3.PoolManager()
        r = http.request('GET', self.jwks_uri)
        if r.status != 200:
            raise JWTClientException('OpenID returned error code %s on jwks_uri: %s' % (r.status, self.jwks_uri))
        self.jwks = JWKSet.from_json(r.data.decode('UTF-8'))



from django.apps import AppConfig
from django.urls import reverse, NoReverseMatch

from django_jwt.settings_utils import get_setting


class DjangoJwtConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_jwt'

    def ready(self):
        url = get_setting('JWT_CLIENT.OPENID2_URL')
        if url == 'local':
            apps = get_setting('INSTALLED_APPS')
            if 'django_jwt.server' not in apps:
                raise Exception('You need to add django_jwt.server into your INSTALLED_APPS in order to deploy the '
                                'OpenId server')
            try:
                reverse('oidc_config')
            except NoReverseMatch:
                Exception('You need to include the django_jwt.urls into your app urls.py file')
        if url == 'fake':
            try:
                reverse('fake_config')
            except NoReverseMatch:
                Exception('You need to include the django_jwt.urls into your app urls.py file')



from django.urls import path

from django_jwt.server import views

urlpatterns = [
    path('jwks', views.JWKsView.as_view(), name="jwks_uri"),
    path('authenticate', views.AuthorizationView.as_view(), name="authorization_endpoint"),
    path('.well-known/openid-configuration', views.OpenIdConfiguration.as_view(), name='oidc_config'),
    # path('userinfo', None, name='userinfo_endpoint'),
    # path('logout', None, name='end_session_endpoint'),
]



import json

from django.contrib.auth import get_user_model
from jwcrypto.jwt import JWT, JWTMissingKey

from django_jwt.openid import OpenId2Info
from django_jwt.settings_utils import get_setting


class JWTAuthentication:
    class JWTException(Exception):
        pass

    @classmethod
    def authenticate_credentials(cls, key):
        jwt = cls.validate_jwt(key)
        claims = json.loads(jwt.claims)
        if not cls.verify_claims(claims):
            raise cls.JWTException()
        user, created = cls.get_or_create_user(profile=claims)
        return user

    @classmethod
    def validate_jwt(cls, token, second=False):
        try:
            jwt = JWT(jwt=token, key=OpenId2Info().jwks)
        except JWTMissingKey:
            if second:
                raise cls.JWTException()
            OpenId2Info().fetch_jwks()
            jwt = cls.validate_jwt(token=token, second=True)
        return jwt

    @classmethod
    def get_or_create_user(cls, profile):
        model = get_user_model()
        translation = get_setting('JWT_CLIENT.RENAME_ATTRIBUTES')
        model_fields = [field.name for field in model._meta.get_fields()]
        defaults = get_setting('JWT_CLIENT.DEFAULT_ATTRIBUTES')
        defaults.update({translation.get(key, key): value for key, value in profile.items()
                         if translation.get(key, key) in model_fields})
        for key, value in defaults.items():
            change_field = getattr(model, 'change_%s' % key, None)
            if change_field is not None:
                defaults[key] = change_field(model, value)
        kwargs = {model.USERNAME_FIELD: defaults[model.USERNAME_FIELD]}
        if get_setting('JWT_CLIENT.CREATE_USER'):
            return model.objects.get_or_create(defaults=defaults, **kwargs)
        try:
            return model.objects.get(**kwargs), False
        except model.DoesNotExist:
            return None, False

    @classmethod
    def verify_claims(cls, claims):
        if get_setting('JWT_CLIENT.CLIENT_ID') not in claims.get('aud', []):
            return False
        url = get_setting('JWT_CLIENT.OPENID2_URL')
        if url in ['local', 'fake']:
            url = get_setting('DEFAULT_DOMAIN') + '/'
        iss = claims.get('iss', None)
        if iss is None:
            return False
        if not url.startswith(iss):
            return False
        return True



from datetime import datetime, timedelta
from Crypto.PublicKey import RSA

import uuid

from Crypto.Util.py3compat import tobytes
from django.conf import settings
from django.contrib.sessions.models import Session
from django.db import models
from jwcrypto.jwk import JWK

from django_jwt.settings_utils import get_setting


def generate_key():
    private_key = RSA.generate(2048)
    return private_key.export_key(passphrase=get_setting('SECRET_KEY'))


class Key(models.Model):
    kid = models.CharField(primary_key=True, default=uuid.uuid4, editable=False, max_length=40)
    private_key = models.BinaryField(editable=False, default=generate_key)
    date = models.DateTimeField(default=datetime.now, editable=False)

    def to_jwk(self):
        jwk = JWK()
        jwk.import_from_pem(self.private_key, kid=str(self.kid), password=tobytes(get_setting('SECRET_KEY')))
        return jwk

    @classmethod
    def get_actual_jwk(cls):
        now = datetime.now()
        expiration_time = timedelta(seconds=get_setting('JWT_SERVER.JWK_EXPIRATION_TIME'))
        try:
            key = cls.objects.get(date__gt=(now - expiration_time))
        except cls.DoesNotExist:
            key = cls()
            key.save()
        return key.to_jwk()

    @classmethod
    def get_jwk_set(cls):
        now = datetime.now()
        expiration_time = timedelta(seconds=get_setting('JWT_SERVER.JWT_EXPIRATION_TIME'))
        cls.objects.filter(date__lt=(now - expiration_time - timedelta(hours=1))).delete()
        keys = cls.objects.all()
        return [key.to_jwk().export_public(as_dict=True) for key in keys]


class WebPage(models.Model):
    id = models.CharField(primary_key=True, default=uuid.uuid4, editable=False, max_length=40, verbose_name='Client id')
    host = models.CharField(unique=True, max_length=200)
    needs_confirmation = models.BooleanField(default=True)

    def __str__(self):
        return self.host


class AttributeWebPage(models.Model):
    web = models.ForeignKey(WebPage, on_delete=models.CASCADE)
    attribute = models.CharField(max_length=100)
    value = models.CharField(max_length=100)
    restrict = models.BooleanField(default=False)

    def __str__(self):
        return 'ID: %s - %s - %s' % (self.id, self.attribute, self.value)


class UserExternalSession(models.Model):
    id = models.CharField(primary_key=True, default=uuid.uuid4, editable=False, max_length=40)
    extra_id = models.CharField(default=uuid.uuid4, unique=True, max_length=40)
    session = models.ForeignKey(Session, on_delete=models.CASCADE)
    web = models.ForeignKey(WebPage, on_delete=models.CASCADE)
    date = models.DateTimeField(auto_now_add=True)


class UserWebPagePermission(models.Model):
    web = models.ForeignKey(WebPage, on_delete=models.CASCADE)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)



from django.http import JsonResponse
from django.shortcuts import render, redirect
from django.urls import reverse

from django_jwt.view_utils import crear_url_amb_jwt, get_jwks, get_sub_jwt


def fake_config(request):
    config = {
        'issuer': request.build_absolute_uri('/'),
        'userinfo_endpoint': request.build_absolute_uri(reverse('fake_userinfo')),
        'authorization_endpoint': request.build_absolute_uri(reverse('fake_login')),
        'jwks_uri': request.build_absolute_uri(reverse('fake_jwks')),
        'response_types_supported': ['id_token'],
        'subject_types_supported': ['public'],
        'id_token_signing_alg_values_supported': ['RS256'],
        'claims_supported': ['sub', 'iss', 'aud', 'exp', 'iat', 'jti', 'scope', 'azp'],
        'require_request_uri_registration': True,
    }
    return JsonResponse(config)


def fake_login(request):
    if request.method == 'GET':
        return render(request, "fake_login.html")
    if request.method == 'POST':
        return redirect(crear_url_amb_jwt(request))


def jwks(request):
    return JsonResponse(get_jwks())


def fake_userinfo(request):
    sub = get_sub_jwt(request.headers.get('Authorization').split(' ')[1])
    return JsonResponse({'sub': sub})
